<?php

namespace FlexibleCouponsVendor\WPDesk\Library\WPCoupons\Settings;

use FlexibleCouponsVendor\WPDesk\Forms\Field\Header;
use FlexibleCouponsVendor\WPDesk\Forms\Field\SelectField;
use FlexibleCouponsVendor\WPDesk\Forms\Field\SubmitField;
use FlexibleCouponsVendor\WPDesk\Forms\Form\FormWithFields;
use FlexibleCouponsVendor\WPDesk\Forms\Resolver\DefaultFormFieldResolver;
use FlexibleCouponsVendor\WPDesk\Library\WPCoupons\CouponsIntegration;
use FlexibleCouponsVendor\WPDesk\Persistence\PersistentContainer;
use FlexibleCouponsVendor\WPDesk\PluginBuilder\Plugin\Hookable;
use FlexibleCouponsVendor\WPDesk\View\Renderer\Renderer;
use FlexibleCouponsVendor\WPDesk\View\Renderer\SimplePhpRenderer;
use FlexibleCouponsVendor\WPDesk\View\Resolver\ChainResolver;
use FlexibleCouponsVendor\WPDesk\View\Resolver\DirResolver;
use FlexibleCouponsVendor\WPDesk\View\Resolver\Resolver;
/**
 * Adds settings to the menu and manages how and what is shown on the settings page.
 *
 * @package WPDesk\Library\FlexibleInvoicesCore\Settings
 */
class SettingsForm implements \FlexibleCouponsVendor\WPDesk\PluginBuilder\Plugin\Hookable
{
    const FIELD_AUTOMATIC_SENDING = 'automatic_sending';
    const EXPIRY_DATE_FORMAT = 'expiry_date_format';
    const ATTACH_COUPON = 'attach_coupon';
    const PRODUCT_PAGE_POSITION = 'coupon_product_position';
    const COUPON_CODE_PREFIX = 'coupon_code_prefix';
    const COUPON_CODE_SUFFIX = 'coupon_code_suffix';
    const COUPON_CODE_LENGTH = 'coupon_code_random_length';
    const PRO_FIELDS = [self::EXPIRY_DATE_FORMAT, self::ATTACH_COUPON, self::PRODUCT_PAGE_POSITION, self::COUPON_CODE_PREFIX, self::COUPON_CODE_SUFFIX, self::COUPON_CODE_LENGTH];
    const MENU_PAGE_URL = 'edit.php?post_type=wpdesk-coupons';
    /**
     * @var string
     */
    const SETTINGS_SLUG = 'fc-settings';
    /**
     * @var PersistentContainer
     */
    private $options_container;
    /**
     * @param PersistentContainer $options_container
     */
    public function __construct(\FlexibleCouponsVendor\WPDesk\Persistence\PersistentContainer $options_container)
    {
        $this->options_container = $options_container;
    }
    private function filter_pro_fields(array $data) : array
    {
        foreach ($data as $data_key => $data_value) {
            if (!\FlexibleCouponsVendor\WPDesk\Library\WPCoupons\CouponsIntegration::is_pro() && \in_array($data_key, self::PRO_FIELDS, \true)) {
                unset($data[$data_key]);
            }
            unset($data_key);
        }
        if (\is_array($data)) {
            return $data;
        }
        return [];
    }
    /**
     * Fires hooks.
     */
    public function hooks()
    {
        \add_action('admin_menu', function () {
            \add_submenu_page(self::MENU_PAGE_URL, \esc_html__('Settings', 'flexible-coupons'), \esc_html__('Settings', 'flexible-coupons'), 'manage_options', self::SETTINGS_SLUG, [$this, 'render_page_action'], 40);
        }, 999);
        \add_action('admin_init', [$this, 'save_settings_action'], 5);
    }
    /**
     * Save POST tab data. Before render.
     *
     * @return void
     */
    public function save_settings_action()
    {
        if (isset($_GET['page']) && $_GET['page'] === self::SETTINGS_SLUG) {
            $settings = $_POST['general'] ?? [];
            if (!empty($settings)) {
                foreach ($this->filter_pro_fields($settings) as $option_key => $option_value) {
                    $this->options_container->set($option_key, $option_value);
                }
            }
        }
    }
    private function get_wc_order_statuses() : array
    {
        $statuses = \wc_get_order_statuses();
        unset($statuses['wc-cancelled'], $statuses['wc-refunded'], $statuses['wc-failed'], $statuses['wc-checkout-draft']);
        return \array_merge([\esc_html__('Do not generate', 'flexible-coupons')], $statuses);
    }
    private function get_fields() : array
    {
        $docs_url = 'pl_PL' === \get_locale() ? 'https://www.wpdesk.pl/docs/flexible-coupons-pro/?utm_source=flexible-coupons-settings&utm_medium=link&utm_campaign=flexible-coupons-docs-link&utm_content=settings#ustawienia' : 'https://www.wpdesk.net/docs/flexible-coupons-pro/?utm_source=flexible-coupons-settings&utm_medium=link&utm_campaign=flexible-coupons-docs-link&utm_content=settings#Settings';
        $fields = [(new \FlexibleCouponsVendor\WPDesk\Forms\Field\Header())->set_label(\esc_html__('General', 'flexible-coupons'))->set_description(
            // translators: %1$s start url, %2$s close url.
            \sprintf(\__('See our %1$sdocumentation%2$s.', 'flexible-coupons'), '<a href="' . \esc_url($docs_url) . '" target="_blank">', '</a>')
        ), (new \FlexibleCouponsVendor\WPDesk\Forms\Field\SelectField())->set_name(self::FIELD_AUTOMATIC_SENDING)->set_label(\esc_html__('Automatically generate coupons', 'flexible-coupons'))->set_options($this->get_wc_order_statuses())->set_description(\esc_html__('If you want the coupon to be generated automatically, select order status. Coupon will be generated and sent automatically when order status is changed to selected status.', 'flexible-coupons'))->set_required(), (new \FlexibleCouponsVendor\WPDesk\Forms\Field\SubmitField())->set_attribute('id', 'save_settings')->set_name('save_settings')->set_label(\esc_html__('Save Changes', 'flexible-coupons'))->add_class('button-primary')];
        return \apply_filters('fcpdf/settings/general/fields', $fields, '');
    }
    /**
     * Render
     *
     * @return void
     */
    public function render_page_action()
    {
        $form = new \FlexibleCouponsVendor\WPDesk\Forms\Form\FormWithFields($this->get_fields(), 'general');
        $form->set_data($this->options_container);
        echo $form->render_form($this->get_renderer());
    }
    /**
     * @return Renderer
     */
    private function get_renderer()
    {
        $chain = new \FlexibleCouponsVendor\WPDesk\View\Resolver\ChainResolver();
        $resolver_list = (array) \apply_filters('fcpdf/settings/template_resolvers', [new \FlexibleCouponsVendor\WPDesk\View\Resolver\DirResolver(__DIR__ . '/Views'), new \FlexibleCouponsVendor\WPDesk\Forms\Resolver\DefaultFormFieldResolver()]);
        foreach ($resolver_list as $resolver) {
            $chain->appendResolver($resolver);
        }
        return new \FlexibleCouponsVendor\WPDesk\View\Renderer\SimplePhpRenderer($chain);
    }
}
